// Тема, в которой обсуждается этот проект, https://clck.ru/JNqKg
// link to the topic that discusses this development: https://clck.ru/JNqKg
//редактирование от 29-07-2021 года 18:00 MCK.

#pragma GCC optimize ("Os")
#include <EEPROM.h>                                              // библиотека энергонезависимой памяти
#include <LiquidCrystal_I2C.h>                                   // библиотека дисплея LCD2004 i2c
LiquidCrystal_I2C lcd(0x27, 20, 4);                              // вместо (0x3f) указываем адрес своего дисплея, например,0x27
#include "ASetting.h"                                            // файл начальных установок

#define WatashiON                                                // настройка на мой дисплей и подключение



//---секция кнопок и пинов подключения---//
#ifdef resist_keyboard                                           // если клавиатура резистивная
#include <ClickBtnLong.h>                                        // библиотека аналоговой (резистивной) клавиатуры 
ClickBtnLong Btn_UP(PinKeyboard, PIN_UP, ABERRATION);            // передаём данные кнопок резистивной клавиатуры в библиотеку ClickBtnLong.h
ClickBtnLong Btn_DOWN(PinKeyboard, PIN_DOWN, ABERRATION);        //
ClickBtnLong Btn_LEFT(PinKeyboard, PIN_LEFT, ABERRATION);        //
ClickBtnLong Btn_RIGHT(PinKeyboard, PIN_RIGHT, ABERRATION);      //
ClickBtnLong Btn_OK(PinKeyboard, PIN_OK, ABERRATION);            //
#else
#include <Cl_do_btn_long.h>                                      // библиотека одиночных кнопок
Cl_do_btn_long Btn_UP(PIN_UP);                                   // передаём данные одиночных кнопок в библиотеку Cl_do_btn_long.h
Cl_do_btn_long Btn_DOWN(PIN_DOWN);                               //
Cl_do_btn_long Btn_LEFT(PIN_LEFT);                               //
Cl_do_btn_long Btn_RIGHT(PIN_RIGHT);                             //
Cl_do_btn_long Btn_OK(PIN_OK);                                   //
#endif

// --------
#define Nst 1
//#define Sst 1
/* //---выбор источника прерывания---// если детектор нуля не используется - закомментировать оба варианта
   и будет использоваться программный таймер (нужна библиотека MsTimer2.h)
   скачать по ссылке https://github.com/PaulStoffregen/MsTimer2 */
//#define SetInterrupt 0                                         // назначение пина детектора нуля ZCC (если используется): подключение к пину 2
//#define SetInterrupt 1                                         // назначение пина детектора нуля ZCC (если используется): подключение к пину 3
#ifndef SetInterrupt                                             // если не используется схема ZCC
#include <MsTimer2.h>                                            // библиотека таймера для работы без схемы ZCC
#endif

//---секция ПИД---//
//---вводим "f2" - доп коэфициент для Pid2TEST2 НИ - это для очень инерционного НИ
float e1, p1, d1, e2, p2, d2, f2;                                // ошибка регулирования, П-составляющая, Д-составляющая для ВИ и НИ соответственно
float integra, integra2, BrTg;                                   // интегральные составляющие ВИ и НИ
#define i_min 0.0                                                // минимум И составляющей
#define i_max 100.0                                              // максимум И составляющей
double  Input1;                                                  // текущая температура(измеренная max6675) ВИ
byte Output1;                                                    // выход ПИД регулятора ВИ
double  Input2;                                                  // текущая температура(измеренная max6675) НИ
byte Output2;                                                    // выход ПИД регулятора НИ

//---секция алгоритма Брезенхема---//
int er1 = 1;                                                     // переменная для расчета переполнения в алгоритме Брезенхема для ВИ
int er2 = 1;                                                     // переменная для расчета переполнения в алгоритме Брезенхема для НИ
int reg1;                                                        // промежуточная переменная для алгоритма Брезенхема для ВИ
int reg2;                                                        // промежуточная переменная для алгоритма Брезенхема для НИ
boolean out1;                                                    // выход импульса управления ВИ алгоритма Брезенхема
boolean out2;                                                    // выход импульса управления НИ алгоритма Брезенхема

struct pr {                                                      //---секция профиля---//  //основные поля профиля
  byte profileSteps;                                             // количество шагов профиля
  int Setpoint2;                                                 // установленная температура(уставка)НИ
  byte pwr_BOTTOM;                                               // максимальная мощность нижнего нагревателя
  byte min_pwr_TOPStep[4];                                       // минимальная мощность верхнего нагревателя
  byte max_pwr_TOPStep[4];                                       // максимальная мощность верхнего нагревателя
  byte rampRateStep[4];                                          // скорости роста температуры по шагам
  byte temperatureStep[4];                                       // заданные температуры для шагов профиля ВИ
  byte dwellTimerStep[4];                                        // установленное время перехода на следующий шаг (полочка на шаге)
  byte kp1;                                                      // пропорциональный коэффициент ПИД для ВИ
  byte ki1;                                                      // интегральный коэффициент ПИД для ВИ
  byte kd1;                                                      // дифференциальный коэффициент ПИД для ВИ
  byte kp2;                                                      // пропорциональный коэффициент ПИД для НИ
  byte ki2;                                                      // интегральный коэффициент ПИД для НИ
  byte kd2;                                                      // дифференциальный коэффициент ПИД для НИ
};
byte SizeProfile = sizeof(pr);                                   // длина поля данных
pr profile;
#define MaxProf 10
byte kf2 = 60;                                                   // доп коэфициент для Pid2TEST2 НИ. //Значение множителя К для Сопротивления изменению t° (f = friction)
byte currentProfile = 1;                                         // текущий профиль
byte editStep = 0;                                               // номер шага профиля который редактируется в данный момент
byte currentStep, BrStep;                                        // текущий шаг профиля
byte Setpoint1;                                                  // установленная температура(уставка) ВИ
// ------------------------

#define ITEMS 9                                                  // Общее количество пунктов в меню
/* массивы строк с именами пунктов */
const char i0[] PROGMEM = " Bottom setting";                     // Каждое имя в своей строке, символ переноса не нужен
const char i1[] PROGMEM = " PreHeater setting";
const char i2[] PROGMEM = " Step 1 Top setting";
const char i3[] PROGMEM = " Step 2 Top setting";
const char i4[] PROGMEM = " Step 3 Top setting";
const char i5[] PROGMEM = " Step 4 Top setting";
const char i6[] PROGMEM = " Bottom PID Long kf2";
const char i7[] PROGMEM = " Bottom PID Setting";
const char i8[] PROGMEM = " Top PID Setting";
const char i9[] PROGMEM = " Parameter 9:";
const char i10[] PROGMEM = " Parameter 10:";
const char i11[] PROGMEM = " Parameter 11:";
const char* const names[] PROGMEM =     {                        // Создаем массив ссылок, обращение к названию пунктов по их номеру
  i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11
};
//---Секция флагов---//
boolean flag = 0;                                                // флаг для фиксации стартовой температуры и начала работы нагревателей
byte x = 1;                                                      // переменная для перехода на нужный шаг при горячей плате
boolean alarmOn = false;                                         // признак предупреждения
boolean updateScreen = true;                                     // признак обновления экрана
boolean EncKey = false;                                          // флаг кнопки энкодера

//---Секция переменных общего назначения---//
int startTemp;                                                   // стартовая температура для профиля, когда НИ вышел на уставку, используется для определения шага профиля с которого начинаем
byte setpointRamp, bottomMaxPwr;                                 // заданная температура по определённому шагу профиля ВИ, связана с заданной скоростью роста температуры
byte curCount = 0;                                               // знакоместо звездочек (они бегают по LCD в режиме RUN)
byte counter;                                                    // счётчик для рампы и счетчик для времени нахождения в шаге
byte Prob;
#ifdef inert_heater
byte inHeater = inert_heater;
#else
byte inHeater = 0;
#endif
int tc1, tc2;                                                    // переменная для хранения текущей температуры на датчиках ВИ и НИ соответственно
int i = 0;
unsigned long previousMillis;                                    // это для счетчиков
unsigned long nextRead1;                                         // переменная для обновления текущей температуры (и складывания данных в буфер для передачи на ПК в IDLE), в режиме пайки в ней считаем ПИД
unsigned long BotStartTime, BrStartTime;

int chast, kluch, Count1, Count2;
double bottomTemp;
boolean TopStart = false, BottomStart = false, HeavyStart = false, HeavyText = false;
boolean FlTone = false, EnClSet = false;
bool F_Alarm, TouDec = false, scrolSet;
byte  BrStepMax;
static int8_t pointer = 0; // Переменная указатель
int8_t pointOld, pointMin, pointMax;
int SP2;
int ArObsi[6];
// 0 - SetQvard включение защиты низа
// 1 -
// 2 - температура старта преднагрева верха(если 0 то по профилю)
// 3 - мощность подаваемая на верх при преднагреве
// 4 - температура старта профиля при инерционном низе (если 0 то профиль стартует сразу)
// 5 - мощность подаваемая на низ при преднагреве
int Pr_Line_M[10][4];
#define TimeSt_M 0
#define TempSt_M 1
#define TimeEnd_M 2
#define TempEnd_M 3
//
char buf[40];                                                    // расширенный буфер, если выводим больше данных
#define GRAPHICS_SAMPLING_TIME 500                               // частота складывания данных в буфер для передачи на ПК

typedef enum REFLOW_STATE : byte {                               //---структура статусов событий---//  // ***** TYPE DEFINITIONS *****
  REFLOW_STATE_IDLE,                                             // экран в режиме простоя, выбор номера профиля
  REFLOW_STATE_MENU_MENU,
  REFLOW_STATE_MENU_STEPS,                                       // меню выбора числа шагов редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_HEAT,                                 // меню выбора уставки НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_PWR,                                  // меню выбора мощности НИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_HEAT,                                    // меню выбора температуры включения преднагрева BИ перед стартом основного профиля
  REFLOW_STATE_MENU_TOP_PWR,                                     // меню выбора мощности подавемой для преднагрева BИ
  REFLOW_STATE_MENU_TOP_PWR_MIN,                                 // меню выбора мин. мощности ВИ в шагах редактируемого профиля
  REFLOW_STATE_MENU_TOP_PWR_MAX,                                 // меню выбора макс. мощности ВИ в шагах редактируемого профиля
  REFLOW_STATE_MENU_STEP_RAMP,                                   // меню выбора скорости роста t° ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_STEP_TARGET,                                 // меню выбора t° уставки ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_STEP_DWELL,                                  // меню выбора времени удержания t° уставки ВИ на конкретном шаге редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_kf2,                                  // меню выбора коэффициента К Сопротивления изменению t° (f = friction) для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_P,                                    // меню выбора коэффициента P для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_I,                                    // меню выбора коэффициента I для НИ редактируемого профиля
  REFLOW_STATE_MENU_BOTTOM_D,                                    // меню выбора коэффициента D для НИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_P,                                       // меню выбора коэффициента P для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_I,                                       // меню выбора коэффициента I для ВИ редактируемого профиля
  REFLOW_STATE_MENU_TOP_D,                                       // меню выбора коэффициента D для ВИ редактируемого профиля
  REFLOW_STATE_MENU_PRE_HEAT,                                    // меню выбора температуры преднагрева НИ после которой старт основного профиля
  REFLOW_STATE_MENU_PRE_PWR,                                     // меню выбора мощности подавемой для преднагрева НИ
  REFLOW_STATE_BRAZE_START,                                      // Старт пайки
  REFLOW_STATE_BRAZE_NOW,                                        // отработка профиля
  REFLOW_STATE_BRAZE_STOP,                                       // Стоп пайки
  REFLOW_STATE_PROFILE_INIT                                      // состояние в IDLE "ИНИЦИАЛИЗАЦИЯ ПРЕДУСТАНОВЛЕННОГО ПРОФИЛЯ"
}
reflowState_t;
reflowState_t reflowState;                                       // переменная статуса пайки
void Skobki(byte str, bool key) {
  if (key) {
    lcd.setCursor(15, str);
    lcd.print(">");
    lcd.setCursor(19, str);
    lcd.print("<");
  }
  else {
    lcd.setCursor(15, str);
    lcd.print(":");
    lcd.setCursor(19, str);
    lcd.print(" ");
  }
}

byte RazrPar(int par) {
  if (par < 10 && par > 0) return (2);
  if (par >= 10 && par < 100) return (1);
  return (0);
}

void diPrin(byte col, byte str, int defr) {
  lcd.setCursor(col, str);
  if (defr == 0) Prob = 2;
  else  Prob = RazrPar(defr);
  if (Prob != 0) {
    for (i = 0; i < Prob; i++ ) lcd.print(" ");
  }
  lcd.print(defr);
}

void PrSteps() {
  lcd.setCursor(Nst, 1);
  lcd.print(F("Steps:   Step :  "));
  lcd.setCursor(8, 1);
  lcd.print(profile.profileSteps);
  lcd.setCursor(18, 1);
  lcd.print(editStep + 1);
}

byte Pid1(double temp, byte ust, byte kP, byte kI, byte kd) {    //----ПИД для ВИ
  byte out = 0;
  static float ed = 0;
  e1 = (ust - temp);                                             // ошибка регулирования
  p1 =  (kP * e1);                                               // П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e1) / 1000; // И составляющая
  d1 = kd * (temp - ed); //Д составляющая
  ed = temp;
  out = (p1 + integra - d1 < profile.min_pwr_TOPStep[currentStep - 1]) ? profile.min_pwr_TOPStep[currentStep - 1] : (p1 + integra - d1 > profile.max_pwr_TOPStep[currentStep - 1]) ? profile.max_pwr_TOPStep[currentStep - 1] : p1 + integra - d1;
  return out;
}

#ifdef LongBottomPid
byte Pid2TEST2(double temp, int ust, byte kP, byte kI, byte kD, byte kF) {   // ПИД для очень инерционного НИ
  byte out = 0;
  static const byte buff_depth = 8;                                          // глубина буфера 4 = 1 секунда, 8 = 2 секунды... MAX 250, увеличение кушает память.
  static float ed[buff_depth];                                               // массив предыдущих измерений ошибки (виден только внутри функции пид)
  static float ef[buff_depth];                                               // массив предыдущих измерений температуры (виден только внутри функции пид)
  //float ed[buff_depth];
  //float ef[buff_depth];
  static byte buff_position;
  e2 = e2 * 0.75 + 0.25 * (ust - temp);                                      // ошибка регулирования Сглаживание на 4 шага
  p2 = (kP * e2 / 10);                                                       // П составляющая
  integra2 = (integra2 < i_min) ? i_min : (integra2 > i_max) ? i_max : integra2 + (kI * e2) / 10000.0; // И составляющая
  d2 = kD * (e2 - ed[buff_position]);                                        // Д составляющая // сравниваем со значением ошибки 8 шагов назад, если ошибка уменьшилась уменьшаем ПИД
  f2 = kF * (temp - ef[buff_position]);                                      // скорость изменения температуры, если быстро растет, гасим пид, если быстро падает - подпинываем
  ed[buff_position] = e2;                                                    // Запись ошибки buff_depth шагов назад
  ef[buff_position] = temp;                                                  // Запись температуры buff_depth шагов назад
  buff_position = (buff_position < buff_depth - 1) ? buff_position + 1 : 0;  // если достигли последнего значения буфера начинаем писать с 0
  //out = (p2 + integra2 + d2 - f2 > 100 ) ? 100 : (p2 + integra2 + d2 - f2  < 0 ) ? 0 : p2 + integra2 + d2 - f2;  // Ограничиваем выходные значения 0-100
  out = (p2 + integra2 + d2 - f2 > profile.pwr_BOTTOM ) ? profile.pwr_BOTTOM : (p2 + integra2 + d2 - f2  < 0 ) ? 0 : p2 + integra2 + d2 - f2;
  return out;
}
#else
byte Pid2(double temp, int ust, byte kP, byte kI, byte kd) {                 // ПИД для "шустрых" НИ на галогенках и им подобных излучателях
  byte out = 0;
  static float ed = 0;
  e2 = (ust - temp);                                                         // ошибка регулирования
  p2 =  (kP * e2);                                                           // П составляющая
  integra2 = (integra2 < i_min) ? i_min : (integra2 > i_max) ? i_max : integra2 + (kI * e2) / 1000; // И составляющая
  d2 = kd * (temp - ed) ;                                                    // Д составляющая
  ed = temp;
  out = (p2 + integra2 - d2 < 0) ? 0 : (p2 + integra2 - d2 > bottomMaxPwr) ? bottomMaxPwr : p2 + integra2 - d2;
  // out = (p2 + integra2 + d2 < u.Profili.min_pwr_BOTTOM) ? u.Profili.min_pwr_BOTTOM : (p2 + integra2 + d2 > bottomMaxPwr) ? bottomMaxPwr : p2 + integra2 + d2;

  return out;
}
#endif

double max6675_read_temp (int ck, int cs, int so) {                          // MAX6675 functions
  char i;
  int tmp = 0;
  digitalWrite(cs, LOW);//cs = 0;                                            // Stop a conversion in progress
  asm volatile (" nop"  "\n\t");
  for (i = 15; i >= 0; i--) {
    digitalWrite(ck, HIGH);
    asm volatile (" nop"  "\n\t");
    if ( digitalRead(so))
      tmp |= (1 << i);
    digitalWrite(ck, LOW);
    asm volatile (" nop"  "\n\t");
  }
  digitalWrite(cs, HIGH);
  if (tmp & 0x4) {
    return (-100);
  } else
    return ((tmp >> 3)) * 0.25;
}

void Dimming() {
  OutPWR_TOP();
  OutPWR_BOTTOM();
}

void OutPWR_TOP() {
  reg1 = Output1 + er1;                                                      // pwr- задание выходной мощности в %, er- ошибка округления
  if (reg1 < 50) {
    out1 = LOW;
    er1 = reg1 ;                                                             // reg- переменная для расчетов
  } else {
    out1 = HIGH;
    er1 = reg1 - 100;
  }
  digitalWrite(RelayPin1, out1);                                             // пин через который осуществляется дискретное управление
}

void OutPWR_BOTTOM() {
  reg2 = Output2 + er2;                                                      // pwr- задание выходной мощности в %, er- ошибка округления
  if (reg2 < 50) {
    out2 = LOW;
    er2 = reg2 ;                                                             // reg- переменная для расчетов
  } else {
    out2 = HIGH;
    er2 = reg2 - 100;
  }
  digitalWrite(RelayPin2, out2);                                             // пин через который осуществляется дискретное управление
}

void loadProfile()  {                                                        // чтение профиля из EEPROM
  EEPROM.get((currentProfile - 1)*SizeProfile, profile);
  for (byte j = 0; j < 6; j++) {
    ArObsi[j] = EEPROM.read(MaxProf * SizeProfile + 2 + j);
  }
}

void SaveProfile()    {                                                      // сохранение текущего профиля
  EEPROM.put((currentProfile - 1)*SizeProfile, profile);
  for (byte j = 0; j < 6; j++) {
    EEPROM.update(MaxProf * SizeProfile + 2 + j, ArObsi[j]);
  }
}

void TempRead() {                                                            // чтение данных от текущих температурах с MAX6675 (преобразователей сигналов термопар)
  if (Input1 <= 0) Input1 = max6675_read_temp (thermoCLK, thermoCStop, thermoSO);
  else Input1 = Input1 * 0.8 + 0.2 * (max6675_read_temp (thermoCLK, thermoCStop, thermoSO));  // термопара "ВЕРХНЕГО" нагревателя
  if (Input2 <= 0) Input2 = max6675_read_temp (thermoCLK, thermoCSbott, thermoSO);
  else Input2 = Input2 * 0.8 + 0.2 * (max6675_read_temp (thermoCLK, thermoCSbott, thermoSO)); // термопара "НИЖНЕГО"  нагревателя
  tc1 = Input1;
  tc2 = Input2;
}

void TempDisplay()  {                                                        // вывод текущих температур на LCD
  if (Input1 < 0) {
    lcd.setCursor(16, 2);
    lcd.print(" Er");
  }
  else {
    diPrin(16, 2, Input1);
  }
  if (Input2 < 0) {
    lcd.setCursor(16, 3);
    lcd.print(" Er");
  }
  else {
    diPrin(16, 3, Input2);
  }
}

void Hello_guys() {                                                          // сообщение приветствия
  lcd.begin();

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(F("brain of IR stations"));
  lcd.setCursor(1, 1);
  lcd.print(F("v1.3.5.7  SPP/irsp"));
  lcd.setCursor(1, 2);
  lcd.print(F("/forum.amperka.ru/"));
  lcd.setCursor(0, 3);
  lcd.print(F("everyone loves cats!"));
}

void SongHello() {                                                           // Мелодия приветствия
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
}

void ClickBeep() {                                                           // звук при нажатии на кнопки
  tone(buzzerPin, 1045);
  delay(100);
  noTone(buzzerPin);
}

void SongAlarm() {                                                           // звук предупреждения
  tone(buzzerPin, 1046);
  delay(100);
  noTone(buzzerPin);
  delay(100);
  tone(buzzerPin, 1046);
  delay(100);
  noTone(buzzerPin);
  delay(100);
  tone(buzzerPin, 1046);
  delay(100);
  noTone(buzzerPin);
  delay(100);
  tone(buzzerPin, 1046);
  delay(100);
  noTone(buzzerPin);
  delay(100);
}

void ChangeState(reflowState_t ChSt, boolean UpScr) {                        // для обновления LCD в разных состояниях программы
  reflowState = ChSt;
  updateScreen = UpScr;
}

void PushOk() {                                                              // обработка короткого нажатия Ok
  if (reflowState == REFLOW_STATE_IDLE) {                                    // если коротко нажать Ok в IDLE
    ChangeState(REFLOW_STATE_BRAZE_START, 1);
  }
  if (reflowState == REFLOW_STATE_MENU_MENU) {
    switch (pointer) {
      case 0: {
          ChangeState(REFLOW_STATE_MENU_STEPS, 1);
          break;
        }
      case 1: {
          ChangeState(REFLOW_STATE_MENU_PRE_HEAT, 1);
          break;
        }
      case 2: {
          ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
          editStep = 0;
          break;
        }
      case 3: {
          ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
          editStep = 1;
          break;
        }
      case 4: {
          ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
          editStep = 2;
          break;
        }
      case 5: {
          ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
          editStep = 3;
          break;
        }
      case 6: {
          ChangeState(REFLOW_STATE_MENU_BOTTOM_kf2, 1);
          break;
        }
      case 7: {
          ChangeState(REFLOW_STATE_MENU_BOTTOM_P, 1);
          break;
        }
      case 8: {
          ChangeState(REFLOW_STATE_MENU_TOP_P, 1);
          break;
        }
    }
  }
}

void PushOkLong() {                                                          // обработка длинного нажатия Ok
  if (reflowState == REFLOW_STATE_IDLE) {                                    // если держим долго Ok в режиме IDLE
    // ChangeState(REFLOW_STATE_MENU_STEPS, 1);                              // то заходим в меню настроек и обновляем LCD
    ChangeState(REFLOW_STATE_MENU_MENU, 1);
  }
  else {
    ChangeState(REFLOW_STATE_MENU_MENU, 1);                                  // иначе - обновляем LCD и возвращаемся в стартовому экрану
    /* SaveProfile();
      lcd.clear();
      lcd.setCursor(2, 1);
      lcd.print(F("Present profile"));
      lcd.setCursor(6, 2);
      lcd.print(F("is Saved!"));
      delay(2000); */
  }
}

void EBTNOk() {                                                              // обработка короткого нажатия Ok
  if (reflowState == REFLOW_STATE_IDLE || reflowState == REFLOW_STATE_MENU_MENU) {   // если коротко нажать Ok в IDLE
    PushOk();
    return;
  }
  if (reflowState == REFLOW_STATE_BRAZE_NOW) {                               // если коротко нажать Ok в IDLE
    PushLeft();
    return;
  }
  if (reflowState == REFLOW_STATE_BRAZE_STOP) {                              // если коротко нажать Ok в IDLE
    PushLeft();
    return;
  } // ChangeState(REFLOW_STATE_IDLE,1);
  if (reflowState >= REFLOW_STATE_MENU_STEPS && reflowState <= REFLOW_STATE_MENU_PRE_PWR ) {
    EncKey = !EncKey;
    return;
  }
}

void EBTNOkLong() {                                                          // обработка длинного нажатия Ok
  if (reflowState == REFLOW_STATE_IDLE) {                                    // если держим долго Ok в режиме IDLE
    ChangeState(REFLOW_STATE_MENU_MENU, 1);                                  // то заходим в меню настроек и обновляем LCD
    return;
  }
  if (reflowState == REFLOW_STATE_MENU_MENU) {
    PushLeftLong();
    return;
  }

  if (reflowState == REFLOW_STATE_BRAZE_STOP) {                              //
    PushLeft();
    return;
  }
  if (reflowState == REFLOW_STATE_BRAZE_NOW) {
    return;
  }
  else {
    ChangeState(REFLOW_STATE_MENU_MENU, 1);
    /* ChangeState(REFLOW_STATE_IDLE, 1);                                    // иначе - обновляем LCD и возвращаемся к стартовому экрану
      SaveProfile();
      lcd.clear();
      lcd.setCursor(2, 1);
      lcd.print(F("Present  profile"));
      lcd.setCursor(6, 2);
      lcd.print(F("is Saved!"));
      delay(2000);  */
  }
}

void PushUp() {                                                              // обработка короткого нажатия Up
  switch (reflowState) {                                                     // всё происходит внутри "машины состояний"
    case REFLOW_STATE_IDLE: {                                                // главное меню
        currentProfile ++;                                                   // увеличить № профиля на 1
        if (currentProfile > MaxProf) currentProfile = 1;                    // если текущий профиль > MaxProf, нужно вернуться к профилю 1
        loadProfile();                                                       // вызов функции загрузки данных профиля из eeprom
        break;
      }
    case REFLOW_STATE_MENU_MENU: {
        if (pointer > 0) {
          pointer--;
        }
        break;
      }
    case REFLOW_STATE_MENU_STEPS: {                                          // устанавливаем количество шагов профиля
        profile.profileSteps ++;                                             // увеличить число шагов профиля на 1
        if (profile.profileSteps >= 5) profile.profileSteps = 1;             // если число шагов > 4, установить число шагов = 1
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                    // устанавливаем температуру "Нижнего Нагревателя"
        profile.Setpoint2++;
        if (profile.Setpoint2 >= 34) profile.Setpoint2 = 8;                  // деленная на 10
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_PWR:  {                                    // устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        profile.pwr_BOTTOM ++;
        if (profile.pwr_BOTTOM >= 100) profile.pwr_BOTTOM = 1;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MIN: {                                    // устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        profile.min_pwr_TOPStep[editStep] ++;
        if (profile.min_pwr_TOPStep[editStep] >= 100) profile.min_pwr_TOPStep[editStep] = 1;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MAX: {                                    // устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        profile.max_pwr_TOPStep[editStep] ++;
        if (profile.max_pwr_TOPStep[editStep] >= 100) profile.max_pwr_TOPStep[editStep] = 1;
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                      // устанавливаем скорость нагрева "Верхним Нагревателем"
        profile.rampRateStep[editStep] ++;
        if (profile.rampRateStep[editStep] >= 31)  profile.rampRateStep[editStep] = 0;      // максимальная скорость роста температуры умноженная на 10
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                    // устанавливаем температуру "Верхнего Нагревателя"
        profile.temperatureStep[editStep] ++;
        if (profile.temperatureStep[editStep] >= 256)  profile.temperatureStep[editStep] = 0;
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                     // устанавливаем время перехода на следующий шаг (длительность шага)
        profile.dwellTimerStep[editStep] ++;
        if (profile.dwellTimerStep[editStep] >= 256)   profile.dwellTimerStep[editStep] = 0;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_kf2: {                                     // настройка "kf2" ПИД нижнего нагревателя
        kf2 ++;
        if (kf2 >= 256)  kf2 = 0;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                       // настройка "P" ПИД нижнего нагревателя
        profile.kp2 ++;
        if (profile.kp2 >= 256)  profile.kp2 = 0;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                       // настройка "I" ПИД нижнего нагревателя
        profile.ki2 ++;
        if (profile.ki2 >= 256)  profile.ki2 = 0;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                       // настройка "D" ПИД нижнего нагревателя
        profile.kd2 ++;
        if (profile.kd2 >= 256)  profile.kd2 = 0;
        break;
      }
    case REFLOW_STATE_MENU_TOP_P: {                                          // настройка "P" ПИД верхнего нагревателя
        profile.kp1 ++;
        if (profile.kp1 >= 256)  profile.kp1 = 0;
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                          // настройка "I" ПИД верхнего нагревателя
        profile.ki1 ++;
        if (profile.ki1 >= 256)  profile.ki1 = 0;
        break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                          // настройка "D" ПИД верхнего нагревателя
        profile.kd1 ++;
        if (profile.kd1 >= 256)  profile.kd1 = 0;
        break;
      }
    case REFLOW_STATE_MENU_PRE_HEAT: {                                       // настройка температурs преднагрева НИ
        ArObsi[4]++;
        if (ArObsi[4] >= 200) ArObsi[4] = 0;
        break;
      }
    case REFLOW_STATE_MENU_PRE_PWR: {                                        // настройка мощности преднагрева НИ
        ArObsi[5]++;
        if (ArObsi[5] >= 100) ArObsi[5] = 0;
        break;
      }
    case REFLOW_STATE_MENU_TOP_HEAT: {                                       // настройка температурs преднагрева НИ
        ArObsi[2]++;
        if (ArObsi[2] >= 200) ArObsi[2] = 0;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR: {                                        // настройка мощности преднагрева НИ
        ArObsi[3]++;
        if (ArObsi[3] >= 100) ArObsi[3] = 0;
        break;
      }
  }
}

void PushDown() {                                                            // обработка короткого нажатия Down
  switch (reflowState) {                                                     // всё происходит внутри "машины состояний"
    case REFLOW_STATE_IDLE: {                                                // главное меню
        currentProfile --;                                                   // уменьшить № профиля на 1
        if (currentProfile <= 0) currentProfile = MaxProf;                   // если № профиля <= 0, установить № профиля MaxProf
        loadProfile();                                                       // вызов функции загрузки данных профиля из eeprom
        break;
      }
    case REFLOW_STATE_MENU_MENU: {
        if (pointer < ITEMS - 1) {
          pointer++;
        }
        break;
      }
    case REFLOW_STATE_MENU_STEPS: {                                          // устанавливаем количество шагов профиля
        profile.profileSteps --;
        if (profile.profileSteps <= 0) profile.profileSteps = 4;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                    // устанавливаем температуру "Нижнего Нагревателя"
        profile.Setpoint2 --;
        if (profile.Setpoint2 <= 7) profile.Setpoint2 = 33;                  // деленная на 10
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_PWR: {                                     // устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        profile.pwr_BOTTOM --;
        if (profile.pwr_BOTTOM <= 0) profile.pwr_BOTTOM = 99;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MIN: {                                    // устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        profile.min_pwr_TOPStep[editStep] --;
        if (profile.min_pwr_TOPStep[editStep] <= 0) profile.min_pwr_TOPStep[editStep] = 99;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MAX: {                                    // устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        profile.max_pwr_TOPStep[editStep] --;
        if (profile.max_pwr_TOPStep[editStep] <= 0)  profile.max_pwr_TOPStep[editStep] = 99;
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                     // устанавливаем скорость нагрева "Верхним Нагревателем"
        profile.rampRateStep[editStep] --;
        if (profile.rampRateStep[editStep] <= 0)  profile.rampRateStep[editStep] = 30;      // минимальная скорость роста температуры умноженная на 10
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                   // устанавливаем температуру "Верхнего Нагревателя"
        profile.temperatureStep[editStep] --;
        if (profile.temperatureStep[editStep] <= 0)  profile.temperatureStep[editStep] = 256;
        if (editStep > 0 && profile.temperatureStep[editStep] < profile.temperatureStep[editStep - 1])
          profile.temperatureStep[editStep] = profile.temperatureStep[editStep - 1];
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                    // устанавливаем время перехода на следующий шаг (длительность шага)
        profile.dwellTimerStep[editStep] --;
        if (profile.dwellTimerStep[editStep] <= 0)  profile.dwellTimerStep[editStep] = 0;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_kf2: {                                    // настройка "kf2" ПИД нижнего нагревателя
        kf2 --;
        if (kf2 < 1)  kf2 = 255;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                      // настройка "P" ПИД нижнего нагревателя
        profile.kp2 --;
        if (profile.kp2 < 1)  profile.kp2 = 255;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                      // настройка "I" ПИД нижнего нагревателя
        profile.ki2 --;
        if (profile.ki2 < 1)  profile.ki2 = 255;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                      // настройка "D" ПИД нижнего нагревателя
        profile.kd2 --;
        if (profile.kd2 < 1)   profile.kd2 = 255;
        break;
      }
    case REFLOW_STATE_MENU_TOP_P: {                                         // настройка "P" ПИД верхнего нагревателя
        profile.kp1 --;
        if (profile.kp1 < 1)  profile.kp1 = 255;
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                         // настройка "I" ПИД верхнего нагревателя
        profile.ki1 --;
        if (profile.ki1 < 1)  profile.ki1 = 255;
        break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                         // настройка "D" ПИД верхнего нагревателя
        profile.kd1 --;
        if (profile.kd1 < 1)  profile.kd1 = 255;
        break;
      }
    case REFLOW_STATE_MENU_PRE_HEAT: {                                      // настройка температурs преднагрева НИ
        ArObsi[4]--;
        if (ArObsi[4] < 0) ArObsi[4] = 200;
        break;
      }
    case REFLOW_STATE_MENU_PRE_PWR: {                                       // настройка мощности преднагрева НИ
        ArObsi[5]--;
        if (ArObsi[5] <= 0) ArObsi[5] = 99;
        break;
      }
    case REFLOW_STATE_MENU_TOP_HEAT: {                                      // настройка температурs преднагрева НИ
        ArObsi[2]--;
        if (ArObsi[2] < 0) ArObsi[2] = 200;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR: {                                       // настройка мощности преднагрева НИ
        ArObsi[3]--;
        if (ArObsi[3] <= 0) ArObsi[3] = 99;
        break;
      }
  }
}

void PushRight() {                                                          // обработка короткого нажатия Right
  switch (reflowState) {
    case REFLOW_STATE_IDLE: {                                               // главное меню, управляем охладителем платы кнопкой ВПРАВО
        digitalWrite(Ext_Fan, HIGH);                                        // включить охладитель платы
        break;
      }
    case REFLOW_STATE_MENU_STEPS: {                                         // устанавливаем количество шагов профиля
        ChangeState(REFLOW_STATE_MENU_BOTTOM_HEAT, 1);                      // обновляем экран
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                   // устанавливаем температуру (уставку) "Нижнего Нагревателя"
        ChangeState(REFLOW_STATE_MENU_BOTTOM_PWR, 1);                       //обновляем экран
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_PWR:  {                                   // устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        ChangeState(REFLOW_STATE_MENU_PRE_HEAT, 1);                         // обновляем экран
        pointer = 1;
        break;
      }
    case REFLOW_STATE_MENU_PRE_HEAT: {                                      // настройка температурs преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_PRE_PWR, 1);
        break;
      }
    case REFLOW_STATE_MENU_PRE_PWR: {                                       // настройка мощности преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_TOP_HEAT, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_HEAT: {                                      // настройка температурs преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_TOP_PWR, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR: {                                       // настройка мощности преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
        pointer = 2;
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                   // устанавливаем температуру "Верхнего Нагревателя"
        ChangeState(REFLOW_STATE_MENU_STEP_RAMP, 1);
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                     // устанавливаем скорость нагрева "Верхним Нагревателем"
        ChangeState(REFLOW_STATE_MENU_STEP_DWELL, 1);
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                    // устанавливаем время перехода на следующий шаг (длительность шага)
        ChangeState(REFLOW_STATE_MENU_TOP_PWR_MIN, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MIN: {                                   // устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        ChangeState(REFLOW_STATE_MENU_TOP_PWR_MAX, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MAX: {                                   // устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        pointer++;
        if (editStep + 1 >= profile.profileSteps) {
          editStep = 0;
          ChangeState(REFLOW_STATE_MENU_BOTTOM_kf2, 1);
        }   else {
          editStep ++;
          ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
        }
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_kf2: {                                    // настройка "kf2" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_BOTTOM_P, 1);
        lcd.setCursor(0, 3);
        lcd.print(F("                    "));
        pointer++;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                      // настройка "P" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_BOTTOM_I, 1);
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                      // настройка "I" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_BOTTOM_D, 1);
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                      // настройка "D" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_TOP_P, 1);
        pointer++;
        break;
      }
    case REFLOW_STATE_MENU_TOP_P: {                                         // настройка "P" ПИД верхнего нагревателя
        ChangeState(REFLOW_STATE_MENU_TOP_I, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                         // настройка "I" ПИД верхнего нагревателя
        ChangeState(REFLOW_STATE_MENU_TOP_D, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_D: {                                         // настройка "D" ПИД верхнего нагревателя
        ChangeState(REFLOW_STATE_MENU_STEPS, 1);
        pointer = 0;
        break;
      }
  }                                                                         // закрываем switch                                                                                        //конец обработки нажатия ВПРАВО
}                                                                           // закрываем PushRight

void PushLeft() {                                                           // обработка короткого нажатия Left
  switch (reflowState) {
    case REFLOW_STATE_IDLE: {                                               // главное меню, управляем охладителем платы кнопкой ВПРАВО
        digitalWrite(Ext_Fan, LOW);                                         // выключить охладитель платы
        break;
      }
    case REFLOW_STATE_MENU_MENU: {
        break;
      }
    case REFLOW_STATE_BRAZE_NOW: ChangeState(REFLOW_STATE_BRAZE_STOP, 1);
      break;
    case REFLOW_STATE_BRAZE_STOP: ChangeState(REFLOW_STATE_IDLE, 1);
      break;
    case REFLOW_STATE_MENU_TOP_D: {                                         // устанавливаем количество шагов профиля
        ChangeState(REFLOW_STATE_MENU_TOP_I, 1);
        pointer = 0;
        break;
      }
    case REFLOW_STATE_MENU_TOP_I: {                                         // устанавливаем температуру "Нижнего Нагревателя"
        ChangeState(REFLOW_STATE_MENU_TOP_P, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_P:  {                                        // устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        ChangeState(REFLOW_STATE_MENU_BOTTOM_D, 1);
        pointer--;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_D: {                                      // устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        ChangeState(REFLOW_STATE_MENU_BOTTOM_I, 1);
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_I: {                                      // устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        ChangeState(REFLOW_STATE_MENU_BOTTOM_P, 1);
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_P: {                                      // устанавливаем скорость нагрева "Верхним Нагревателем"
        ChangeState(REFLOW_STATE_MENU_BOTTOM_kf2, 1);
        pointer--;
        lcd.setCursor(0, 3);
        lcd.print(F("                    "));
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_kf2: {                                    // настройка "kf2" ПИД нижнего нагревателя
        editStep = profile.profileSteps - 1;
        ChangeState(REFLOW_STATE_MENU_TOP_PWR_MAX, 1);
        pointer--;
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MAX: {                                   // настройка "I" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_TOP_PWR_MIN, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR_MIN: {                                   // настройка "D" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_STEP_DWELL, 1);
        break;
      }
    case REFLOW_STATE_MENU_STEP_DWELL: {                                    // устанавливаем температуру "Верхнего Нагревателя"
        ChangeState(REFLOW_STATE_MENU_STEP_RAMP, 1);
        break;
      }
    case REFLOW_STATE_MENU_STEP_RAMP: {                                     // настройка "P" ПИД нижнего нагревателя
        ChangeState(REFLOW_STATE_MENU_STEP_TARGET, 1);
        break;
      }
    case REFLOW_STATE_MENU_STEP_TARGET: {                                   // устанавливаем время перехода на следующий шаг (длительность шага)
        pointer--;
        if (editStep - 1 < 0) {
          editStep = 0;
          ChangeState(REFLOW_STATE_MENU_TOP_PWR, 1);
        }  else {
          editStep --;
          ChangeState(REFLOW_STATE_MENU_TOP_PWR_MAX, 1);
        }
        break;
      }
    case REFLOW_STATE_MENU_TOP_PWR: {                                       // настройка мощности преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_TOP_HEAT, 1);
        break;
      }
    case REFLOW_STATE_MENU_TOP_HEAT: {                                      // настройка температурs преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_PRE_PWR, 1);
        break;
      }
    case REFLOW_STATE_MENU_PRE_PWR: {                                       // настройка мощности преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_PRE_HEAT, 1);
        break;
      }
    case REFLOW_STATE_MENU_PRE_HEAT: {                                      // настройка температурs преднагрева НИ
        ChangeState(REFLOW_STATE_MENU_BOTTOM_PWR, 1);
        pointer--;
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_PWR: {                                    // настройка "P" ПИД верхнего нагревателя
        ChangeState(REFLOW_STATE_MENU_BOTTOM_HEAT, 1);
        break;
      }
    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                   // настройка "I" ПИД верхнего нагревателя
        ChangeState(REFLOW_STATE_MENU_STEPS, 1);
        break;
      }
    case REFLOW_STATE_MENU_STEPS: {                                         // настройка "D" ПИД верхнего нагревателя
        ChangeState(REFLOW_STATE_MENU_TOP_D, 1);
        pointer = 8;
        break;
      }
  }                                                                         // закрываем switch                                                                                        //конец обработки нажатия LEFT
}                                                                           // закрываем PushLeft

void PushLeftLong() {                                                       // обработка длинного нажатия Left
  if (reflowState == REFLOW_STATE_IDLE) {                                   // держим долго Left в режиме IDLE
    reflowState = REFLOW_STATE_PROFILE_INIT;                                // и запускается инициализация предустановленного профиля
  }
  if (reflowState == REFLOW_STATE_MENU_MENU) {
    ChangeState(REFLOW_STATE_IDLE, 1);                                      // иначе - обновляем LCD и возвращаемся в стартовому экрану
    SaveProfile();
    lcd.clear();
    lcd.setCursor(2, 1);
    lcd.print(F("Present profile"));
    lcd.setCursor(6, 2);
    lcd.print(F("is Saved!"));
    delay(2000);
  }
}

void PushUpLong() {                                                         // обработка длинного нажатия Up (пока в резерве)

}

void PushDownLong() {                                                       // обработка длинного нажатия Down (пока в резерве)

}

void PushRightLong() {                                                      // обработка длинного нажатия Right (пока в резерве)

}

void Pr_Line_Load(int i_i, int X_St, int Y_St, int X_En, int Y_En) {
  Pr_Line_M[i_i][0] =  X_St;
  Pr_Line_M[i_i][1] =  Y_St;
  Pr_Line_M[i_i][2] =  X_En;
  Pr_Line_M[i_i][3] =  Y_En;
}

double M_tg_Line(int i_i) {
  double Tg_ugla;
  Tg_ugla = 0;
  if (Pr_Line_M[i_i][2] != Pr_Line_M[i_i][0]) {
    Tg_ugla = Pr_Line_M[i_i][3] - Pr_Line_M[i_i][1];
    Tg_ugla = Tg_ugla / (Pr_Line_M[i_i][2] - Pr_Line_M[i_i][0]);
  }
  // Serial.print("Tg - "); Serial.println(Tg_ugla);
  return (Tg_ugla);
}

void InitLine(int t_start) {         //  рисуем профиль на графике
  int Temp_Start, Temp_End;
  int Time_Start = 0, Time_End;
  int Rt, SetP2;
  byte Rc = 8;
  // заполняем график температуры НИЗа
  Temp_Start = t_start;
  SetP2 = 10 * profile.Setpoint2;
  Temp_End = SetP2;
  Time_End = Time_Start + 10 * (Temp_End - t_start) / Rc;
  Pr_Line_Load (0, Time_Start, Temp_Start, Time_End, SetP2);                // первая линия для низа

  Rt = Time_End;
  Time_End += 5;                // через 5 сек включаем верх
  Time_Start = Time_End;
  Temp_Start = SetP2;

  // рисуем график температуры ВЕРХа по шагам
  BrStepMax = 8;
  for (int i = 0; i < 4; i++) {                                             // определяем количество шагов
    if (profile.rampRateStep[i] == 0) {
      BrStepMax = i * 2;
      break;
    }
    Temp_End = profile.temperatureStep[i] - Temp_Start;
    if (Temp_End < 0) Temp_End = - Temp_End;
    Time_End = Time_Start + (10 * Temp_End) / profile.rampRateStep[i];
    Pr_Line_Load ((i + 1) * 2, Time_Start, Temp_Start, Time_End, profile.temperatureStep[i]);

    Pr_Line_Load( (i + 1) * 2 + 1, Time_End, profile.temperatureStep[i], Time_End + profile.dwellTimerStep[i], profile.temperatureStep[i]);
    Time_End += profile.dwellTimerStep[i];
    Temp_Start = profile.temperatureStep[i];
    Time_Start = Time_End;
  }
  Pr_Line_Load(1, Rt, SetP2, Time_Start, SetP2);

  /*   for (i=0; i<10; i++) {                                               // вывод в порт массива
        for (int j=0; j<4; j++) {
         Serial.print(Pr_Line_M[i][j]);
         Serial.print("  ");
        }
        Serial.println("");
       } */
}

void printMenuItem(uint8_t num) {                                           // Функция для печати строки из progmem
  char buffer[21];                                                          // Буфер на полную строку

  uint16_t ptr = pgm_read_word(&(names[num]));                              // Получаем указатель на первый символ строки
  uint8_t i = 0;                                                            // Переменная - счетчик
  do {                                                                      // Начало цикла
    buffer[i] = (char)(pgm_read_byte(ptr++));                               // Прочитать в буфер один символ из PGM и подвинуть указатель на 1
  } while (buffer[i++] != NULL);                                            // Если это не конец строки - вернуться в начало цикла
  if (i > 2) {
    for (i--; i < 20; i++) buffer[i] = char(0x20);
    buffer[i] = NULL;
  }
  lcd.print(buffer);                                                        // Вывод готовой строки с переносом на следующую
}

void PrLcdProf () {
  lcd.clear();
  lcd.setCursor(Nst, 0);
  lcd.print(F("Profile  Edit : "));
  lcd.setCursor(18, 0);
  lcd.print(currentProfile);
}

void setup() {                                                              // начальные установки
  Serial.begin(9600);
#ifdef resist_keyboard
  pinMode(PinKeyboard, INPUT);
#else
  pinMode (PIN_UP, INPUT_PULLUP);                                           // подключен подтягивающий резистор
  pinMode (PIN_DOWN, INPUT_PULLUP);                                         // подключен подтягивающий резистор
  pinMode (PIN_LEFT, INPUT_PULLUP);                                         // подключен подтягивающий резистор
  pinMode (PIN_RIGHT, INPUT_PULLUP);                                        // подключен подтягивающий резистор
  pinMode (PIN_OK, INPUT_PULLUP);                                           // подключен подтягивающий резистор
#endif

  pinMode(thermoCStop, OUTPUT);                                             // задаём состояние пинов, к которым подключены MAX6675
  pinMode(thermoCSbott, OUTPUT);                                            //
  pinMode(thermoCLK, OUTPUT);                                               //
  pinMode(thermoSO, INPUT);                                                 //
  pinMode(buzzerPin, OUTPUT);                                               // устанавливаем пин пищалки на ВЫХОД
  pinMode (Int_Fan, OUTPUT);                                                // устанавливаем пины охладителей на ВЫХОД
  pinMode (Ext_Fan, OUTPUT);                                                //
  pinMode(RelayPin1, OUTPUT);                                               // задаём состояние пинов управления выходами
  pinMode(RelayPin2, OUTPUT);                                               //

#ifdef SetEncoder                                                           // добавил, чтобы корректно переключаться между устройствами ввода
  pinMode(Enc_Clk, INPUT);                                                  // задаем состояние пинов энкодера
  pinMode(Enc_DT, INPUT);                                                   //
  encoder.setPosition(LastPos);
#endif

  Hello_guys();                                                             // сообщение приветствия
  SongHello ();                                                             // Мелодия приветствия
  delay(3000);                                                              // показываем заставку пока стабилизируются MAX6675
  lcd.clear();                                                              // очищаем LCD
  loadProfile();                                                            // вызов функции loadProfile для загрузки данных профиля из eeprom
  nextRead1 = millis();                                                     // запускаем чтение с термопар

  /*настраиваем работу с прерыванием: LOW вызывает прерывание, когда на порту LOW
    CHANGE прерывание вызывается при смене значения на порту, с LOW на HIGH и наоборот
    RISING прерывание вызывается только при смене значения на порту с LOW на HIGH
    FALLING прерывание вызывается только при смене значения на порту с HIGH на LOW */
#ifdef SetInterrupt                                                         // если используется схема ZCC
  attachInterrupt(SetInterrupt, Dimming, RISING);                           // настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
#else
  MsTimer2::set(10, Dimming);                                               // 10ms period (настройка таймера)
  MsTimer2::start();                                                        // если прерывания по таймеру, а не от схемы ZCC
#endif
}

void loop() {                                                               //---основной цикл программы---//
  unsigned long currentMillis = millis();
  float out_float;                                                          // вывод на LCD дробных чисел
  //Serial.println(analogRead(PinKeyboard));

  Btn_UP.run(&PushUp, &PushUpLong);                                         // указываем обработчики для кнопки Up
  Btn_DOWN.run(&PushDown, &PushDownLong);                                   // указываем обработчики для кнопки Down
  Btn_LEFT.run(&PushLeft, &PushLeftLong);                                   // указываем обработчики для кнопки Left
  Btn_RIGHT.run(&PushRight, &PushRightLong);                                // указываем обработчики для кнопки Right
  Btn_OK.run(&PushOk, &PushOkLong);                                         // указываем обработчики для кнопки Ok
  /* при подключении энкодера: крутилка вправо-влево это выбор параметра,
    жмем на кнопку энкодера переключаемся на изменение параметра,
    еще раз на кнопку и переходим на выбор параметра,
    длинное нажатие - запись в память и выход в главное меню. */

#ifdef SetEncoder
  EBTN_ok.run(&EBTNOk, &EBTNOkLong);                                        // опрашиваем кнопку энкодера

  encoder.tick();                                                           // читаем енкодер
  NewPos = encoder.getPosition();
  //Serial.print(NewPos); Serial.print("  "); Serial.print(LastPos); Serial.print("  ");  Serial.println(reflowState);
  if (LastPos != NewPos) {
    //Serial.println(NewPos);
    if (reflowState == REFLOW_STATE_IDLE || reflowState == REFLOW_STATE_MENU_MENU) {
      if (NewPos > LastPos)  PushUp();
      if (NewPos < LastPos)  PushDown();
      encoder.setPosition(LastPos);
      return;
    }
    if (EncKey) {
      if (NewPos > LastPos)  PushUp();
      if (NewPos < LastPos)  PushDown();
    } else {
      if (NewPos > LastPos)  PushRight();
      if (NewPos < LastPos)  PushLeft();
    }
    encoder.setPosition(LastPos);
    //  break;
  }
#endif

  switch (reflowState) {
    case REFLOW_STATE_IDLE: {                                               // главное меню
        if (updateScreen) {                                                 // Настройка экрана в режиме ожидания
          lcd.clear();
          lcd.setCursor(1, 0);
          lcd.print(F("Station state:IDLE "));
          lcd.setCursor(0, 1);
          lcd.print(F("Profile:"));
          lcd.setCursor(12, 1);
          lcd.print(F("Step: 1"));
          lcd.setCursor(1, 2);
          lcd.print(F("TH  SP:"));
          lcd.setCursor(13, 2);
          lcd.print(F("PV:"));
          lcd.setCursor(1, 3);
          lcd.print(F("BH  SP:"));
          lcd.setCursor(13, 3);
          lcd.print(F("PV:"));
          updateScreen = false;
          TopStart = false;                                                 // ВИ отключен
          currentStep = 1;                                                  // шаг установлен №1
          counter = 0;                                                      // счетчик обнулили
          setpointRamp = 0;                                                 // скорость роста t°C обнулили
          x = 1;                                                            // устанавливаем переменную в исходное состояние
          flag = 0;                                                         // после отановки профиля сбрасываем флаг
          Output1 = 0;                                                      // выход ПИД ВИ обнулили
          Output2 = 0;                                                      // выход ПИД НИ обнулили
        }

        if (millis() > nextRead1) {                                         // чтение и вывод на LCD температур, подготовка данных для ПК
          nextRead1 = millis() + SENSOR_SAMPLING_TIME;                      // устраняем колебания датчиков температуры
          TempRead();

          if (kluch == 4) {
            kluch = 0;
#ifdef SetConnectPC
#ifdef SerialPortPlotter                                                    // готовим данные для вывода на ПК
            sprintf (buf, "$%03d %03d %03d %03d;", inttput1), int(Output2), tc1, tc2);       // стандартный вывод данных
#else
            sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
#endif
            Serial.print(buf);
#endif
          }
          if (kluch == 2) TempDisplay();
          kluch++;
        }

        diPrin(8, 1, currentProfile);
        diPrin(8, 2, profile.temperatureStep[0]);
        diPrin(8, 3, profile.Setpoint2 * 10);
      } break;

    case REFLOW_STATE_MENU_MENU: {
        if (updateScreen) {
          lcd.clear();
          pointOld = ITEMS + 2;
          pointMin = 0; pointMax = 3;
          scrolSet = true;
          updateScreen = false;
        }
        if (pointOld != pointer)  {
          if (pointer > pointMax) {
            pointMax = pointer;
            pointMin = pointer - 3;
            scrolSet = true;
          }
          if (pointer < pointMin) {
            pointMax = pointer + 3;
            pointMin = pointer;
            scrolSet = true;
          }
          // Serial.print(pointMin); Serial.print("  "); Serial.println(pointMax);
          if (scrolSet) {
            for (uint8_t i = 0; i < 4; i++) {                               // Цикл, выводящий 4 пунктов на дисплей
              lcd.setCursor(0, i);
              printMenuItem(pointMin + i);                                  // Выводим пункты + скроллинг при достижении последней строки указателем
            }
            scrolSet = false;
          }
          lcd.setCursor(0, pointOld - pointMin);                            // Указываем на параметр
          lcd.print(" ");
          pointOld = pointer;
          lcd.setCursor(0, pointer - pointMin);                             // Указываем на параметр
          lcd.print(">");
        }
      } break;

    case REFLOW_STATE_PROFILE_INIT: {                                       // инициализация тестового профиля
        /* 4 шага, уставка НИ 160° (записываем поделив на 10, показываем как обычно),
           BOTTOM PWR 50%,
           min TOP PWR по шагам 10%,10%,10%,10%
           max TOP PWR по шагам 99%,99%,99%,99%
           скорости роста t° по шагам 10,3,3,6
           уставки ВИ по шагам 180,190,200,225
           длительности полок по шагам 10,15,15,30
           P.I.D BOTTOM 8,20,40, P.I.D TOP 10,15,40 */

        profile = {4, 16, 50, 10, 10, 10, 10,
                   99, 99, 99, 99, 10, 3, 3, 6,
                   180, 190, 200, 225,
                   10, 15, 15, 30,
                   10, 15, 40, 8, 20, 40
                  };
        SaveProfile();
        tone(buzzerPin, 800, 200);                                         // звуковой сигнал при сбросе профиля
        lcd.clear();
        lcd.setCursor(1, 1);
        lcd.print(F("the preset profile"));
        lcd.setCursor(3, 2);
        lcd.print(F("is a uploaded!"));
        delay(2000);
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      } break;

    case REFLOW_STATE_MENU_STEPS: {                                        // устанавливаем количество шагов профиля
        if (updateScreen) {
          PrLcdProf();
          lcd.setCursor(0, 1);
          lcd.print(F("    All Steps  : "));
          updateScreen = false;
        }
        Skobki(1, EncKey);
        lcd.setCursor(18, 1);
        lcd.print(profile.profileSteps);
      } break;

    case REFLOW_STATE_MENU_BOTTOM_HEAT: {                                  // устанавливаем температуру "Нижнего Нагревателя"
        if (updateScreen) {
          lcd.setCursor(Nst, 2);
          lcd.print(F("Bottom Heat"));
          lcd.setCursor(16, 2);
          lcd.print("  ");
          updateScreen = false;
        }
        Skobki(2, EncKey);
        SP2 = profile.Setpoint2 * 10;                                      // множим уставку НИ на 10
        diPrin(16, 2, SP2);
      } break;

    case REFLOW_STATE_MENU_BOTTOM_PWR: {                                   // устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
        if (updateScreen) {
          lcd.setCursor(0, 1);
          lcd.print(F("   All Steps   :  "));
          lcd.setCursor(18, 1);
          lcd.print(profile.profileSteps);
          lcd.setCursor(Nst, 2);
          lcd.print(F("Bottom  Power"));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.pwr_BOTTOM);
      } break;

    case REFLOW_STATE_MENU_TOP_PWR_MIN: {                                  // устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        if (updateScreen) {
          PrSteps();
          lcd.setCursor(Nst, 2);
          lcd.print(F("Top min Power"));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.min_pwr_TOPStep[editStep]);
      } break;

    case REFLOW_STATE_MENU_TOP_PWR_MAX: {                                  // устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
        if (updateScreen) {
          lcd.setCursor(Nst, 1);
          lcd.print(F("Steps:   Step : "));
          lcd.setCursor(8, 1);
          lcd.print(profile.profileSteps);
          lcd.setCursor(18, 1);
          lcd.print(editStep + 1);
          lcd.setCursor(Nst, 2);
          lcd.print(F("Top max Power"));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.max_pwr_TOPStep[editStep]);
      } break;

    case REFLOW_STATE_MENU_STEP_RAMP: {                                    // устанавливаем скорость нагрева "Верхним Нагревателем"
        if (updateScreen) {
          lcd.setCursor(18, 1);
          lcd.print(editStep + 1);
          lcd.setCursor(Nst, 2);
          lcd.print(F("Ramp  speed   :"));
          updateScreen = false;
        }
        Skobki(2, EncKey);                                                 // не менять на подрограммку
        lcd.setCursor(16, 2);
        out_float = profile.rampRateStep[editStep] / 10.0;
        lcd.print(out_float, 1);
      } break;

    case REFLOW_STATE_MENU_STEP_TARGET: {                                  // устанавливаем температуру "Верхнего Нагревателя"
        if (updateScreen) {
          PrLcdProf();
          PrSteps();
          lcd.setCursor(Nst, 2);
          lcd.print(F("Temp target   "));
          lcd.print(profile.temperatureStep[editStep]);
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.temperatureStep[editStep]);
      } break;;

    case REFLOW_STATE_MENU_STEP_DWELL: {                                   // устанавливаем время перехода на следующий шаг (длительность шага)
        if (updateScreen) {
          lcd.setCursor(Nst, 1);
          lcd.print(F("Steps:   Step : "));
          lcd.setCursor(8, 1);
          lcd.print(profile.profileSteps);
          lcd.setCursor(18, 1);
          lcd.print(editStep + 1);
          lcd.setCursor(Nst, 2);
          lcd.print(F("Dwell  time      "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.dwellTimerStep[editStep]);
      } break;

    case REFLOW_STATE_MENU_BOTTOM_kf2: {                                  // настройка "kf2" ПИД нижнего нагревателя
        if (updateScreen) {
          PrLcdProf();
          lcd.setCursor(Nst, 1);
          lcd.print(F(" Bottom Heater    "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID long kf2    "));
          lcd.setCursor(7, 2);
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, kf2);
      } break;;

    case REFLOW_STATE_MENU_BOTTOM_P: {                                    // настройка "P" ПИД нижнего нагревателя
        if (updateScreen) {
          PrLcdProf();
          lcd.setCursor(Nst, 1);
          lcd.print(F(" Bottom Heater    "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID Proport.    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.kp2);
      } break;

    case REFLOW_STATE_MENU_BOTTOM_I: {                                    // настройка "I" ПИД нижнего нагревателя
        if (updateScreen) {
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID Integral    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.ki2);
      } break;

    case REFLOW_STATE_MENU_BOTTOM_D: {                                    // настройка "D" ПИД нижнего нагревателя
        if (updateScreen) {
          lcd.setCursor(Nst, 1);
          lcd.print(F(" Bottom Heater    "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID  Differ.    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.kd2);
      } break;

    case REFLOW_STATE_MENU_TOP_P: {                                       // настройка "P" ПИД верхнего нагревателя
        if (updateScreen) {
          PrLcdProf();
          lcd.setCursor(Nst, 1);
          lcd.print(F("  Top Heater "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID Proport.    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.kp1);
      } break;

    case REFLOW_STATE_MENU_TOP_I: {                                       // настройка "I" ПИД верхнего нагревателя
        if (updateScreen) {
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID Integral    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.ki1);
      } break;

    case REFLOW_STATE_MENU_TOP_D: {                                       // настройка "D" ПИД верхнего нагревателя
        if (updateScreen) {
          lcd.setCursor(Nst, 1);
          lcd.print(F("  Top  Heater     "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("PID  Differ.    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, profile.kd1);
      } break;

    case REFLOW_STATE_MENU_PRE_HEAT: {                                    // настройка температурs преднагрева НИ
        if (updateScreen) {
          PrLcdProf();
          lcd.setCursor(Nst, 1);
          lcd.print(F(" Bottom PreHeater  "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("Temp PreHeat    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, ArObsi[4]);

      } break;

    case REFLOW_STATE_MENU_PRE_PWR: {                                     // настройка мощности преднагрева НИ
        if (updateScreen) {
          //  lcd.setCursor(Nst, 1);
          //  lcd.print(F(" Bottom PreHeater  "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("Power PreHeat   "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, ArObsi[5]);

      } break;

    case REFLOW_STATE_MENU_TOP_HEAT: {                                    // меню выбора температуры преднагрева НИ после которой старт основного профиля
        if (updateScreen) {
          lcd.setCursor(Nst, 1);
          lcd.print(F("  Top PreHeater   "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("Temp PreHeat    "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, ArObsi[2]);
      } break;

    case REFLOW_STATE_MENU_TOP_PWR:  {                                    // меню выбора мощности подавемой для преднагрева НИ
        if (updateScreen) {
          PrLcdProf();
          lcd.setCursor(Nst, 1);
          lcd.print(F("  Top PreHeater  "));
          lcd.setCursor(Nst, 2);
          lcd.print(F("Power PreHeat   "));
          updateScreen = false;
        }
        Skobki(2, EncKey);
        diPrin(16, 2, ArObsi[3]);


      } break;

    case REFLOW_STATE_BRAZE_START:  {                                     // Старт пайки все начальные установки
        if (updateScreen) {
          lcd.setCursor(0, 0);
          lcd.print(F("Start            "));
          updateScreen = false;
        }
        if (HeavyBottom) {
          HeavyStart = true;
          tc2 = ArObsi[4];
        }
        tone(buzzerPin, 1045, 500);                                       // звуковой сигнал при старте профиля
        delay(200);
        tone(buzzerPin, 1045, 500);                                       // звуковой сигнал при старте профиля
        delay(150);
        InitLine(tc2);
        ChangeState(REFLOW_STATE_BRAZE_NOW, 0);
        TopStart = false;
        BottomStart = false;
        HeavyText = false;
        FlTone = false;
        F_Alarm = false;
        BrStartTime = millis();
        BotStartTime = millis();
        Count1 = 0; Count2 = 0; BrStep = 0;
        BrTg = M_tg_Line(BrStep);
        //Serial.print("Tg - "); Serial.println(BrTg);
        sprintf (buf, "SYNC\r\n");
        Serial.println(buf);

#ifdef SetInterrupt
        attachInterrupt(SetInterrupt, Dimming, RISING); // настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
#else
        MsTimer2::set(10, Dimming); // 10 ms period
        MsTimer2::start();
#endif

      }  break;

    case REFLOW_STATE_BRAZE_NOW:    {                                     // процесс пайки графики
        if (millis() > nextRead1)   {                                     // график температуры nextRead1
          nextRead1 = millis() + SENSOR_SAMPLING_TIME;
          TempRead();                                                     // считываем температуру

          if (ArObsi[4] != 0 && HeavyStart) {
            if (!HeavyText) {                                             // запускаем преднагрев низа
              HeavyText = true;
              lcd.setCursor(0, 0);
              lcd.print(F("Bottom PreHeat ON  "));                        // НИЖНИЙ НАГРЕВАТЕЛЬ ВКЛЮЧЕН
              diPrin(8, 3, ArObsi[4]);
              //             myGLCD.textRus(F(" ПРЕДВАРИТЕЛЬНЫЙ НАГРЕВ НИЗА  "), 20, 302);
              bottomTemp = ArObsi[4];
              bottomMaxPwr = ArObsi[5];
            }
            if (Input2 < ArObsi[4]) {                                     // если температура платы меньше
              TempDisplay();                                              // выводим температуру на дисплей
              Count1++;
              kluch =  Count1 % 4;
            }
            else {                                                        // если больше выходим из режима преднгрева
              HeavyStart = false;
              bottomMaxPwr = profile.pwr_BOTTOM;
              diPrin(8, 3, profile.Setpoint2 * 10);
              Count1 = 0;
              tone(buzzerPin, 1105, 200);                                 // звуковой сигнал
            }
          }
          else {                                                          // начало основного профиля
            if (BrStep <= BrStepMax) {                                    // если не конец профиля
              Count1++;
              chast = Count1 / 4;
              kluch =  Count1 % 4;
              /*
                           // #define TimeSt_M 0
                           // #define TempSt_M 1
                           // #define TimeEnd_M 2
                           // #define TempEnd_M 3
              */
              if (BrStep == 0) {                                          // включение Нижнего подогрева
                if (chast < Pr_Line_M[BrStep][TimeEnd_M]) {               // рост температуры
                  bottomTemp = Pr_Line_M[BrStep][TempSt_M] + BrTg * Count1 / 4;
                }
                else bottomTemp = Pr_Line_M[0][TempEnd_M];                // удержание
                if (bottomTemp != 0 && !BottomStart) {
                  BottomStart = true;
                  lcd.setCursor(0, 0);
                  lcd.print(F("Start  Bottom ON "));                      // НИЖНИЙ НАГРЕВАТЕЛЬ ВКЛЮЧЕН
                }
                if (ArObsi[2] != 0) {                                     // включение преднагрева ВЕРХА
                  if (Input1 > ArObsi[2] && !TopStart && BrStepMax != 0)  { // ПРЕДВАРИТЕЛЬНЫЙ НАГРЕВ ВЕРХА

                    Output1 = ArObsi[3];  // u.Profili.min_pwr_TOPStep[0]
                    if (!FlTone) {
                      FlTone = true;
                      lcd.setCursor(0, 0);
                      lcd.print(F(" Top  PreHeat ON  "));
                      //                        myGLCD.textRus(F(" ПРЕДВАРИТЕЛЬНЫЙ НАГРЕВ ВЕРХА    "), 20, 302);
                      tone(buzzerPin, 1105, 200);                         // звуковой сигнал
                    }
                  }
                }
                if (chast > Pr_Line_M[BrStep + 2][TimeSt_M] && BrStepMax >= 2) { // пора ли включать ВЕРХ
                  TopStart = true;
                  BrStep = + 2;
                  currentStep = BrStep / 2;
                  lcd.setCursor(0, 0);
                  lcd.print(F("Start Top ON    "));
                  //                    myGLCD.textRus(F(" ВЕРХНИЙ НАГРЕВАТЕЛЬ ВКЛЮЧЕН   "), 20, 302);
                  BrTg = M_tg_Line(BrStep);
                  Count2 = 0;
                  tone(buzzerPin, 1105, 200);                             // звуковой сигнал
                  return;
                }
              }
              else {                                                      // включение Верхнего нагревателя
                if (chast <= Pr_Line_M[BrStep][TimeEnd_M] ) {
                  //  Serial.print(BrStep); Serial.print(" "); Serial.println(BrTg);
                  Count2++;
                  Setpoint1 = Pr_Line_M[BrStep][TempSt_M] + BrTg * Count2 / 4; // рост температуры
                }
                else Setpoint1 = Pr_Line_M[BrStep][TempEnd_M];            // удержание

                if (chast > Pr_Line_M[BrStep + 1][TimeEnd_M]) {           // пора ли переходить на следующий шаг
                  BrStep += 2;
                  currentStep = BrStep / 2;
                  BrTg = M_tg_Line(BrStep);
                  Count2 = 0;
                  if (currentStep < 5) {
                    lcd.setCursor(18, 1);
                    lcd.print(currentStep);
                    diPrin(8, 2, profile.temperatureStep[currentStep - 1]);
                  }

                  tone(buzzerPin, 1105, 200);                             // звуковой сигнал
                }
              }
            }
            else  {
              ChangeState(REFLOW_STATE_BRAZE_STOP, 1);                    // завершение пайки
            }
          }
          if (millis() - BotStartTime < 3000) Output2 = 5;                // первые 3 сек мощность низа =5%
          else  Output2 = Pid2(Input2, bottomTemp, profile.kp2, profile.ki2, profile.kd2);
          if (TopStart == true) Output1 = Pid1(Input1, Setpoint1, profile.kp1, profile.ki1, profile.kd1);

                                                                          // вывод информации на экран и в монитор РС
          if (kluch == 0) {                                               // выводим показания секундомера
            TempDisplay();
            diPrin(17, 0, chast);
          }
          if (kluch == 1) {                                               // рисуем графики
            // WinStartTime = micros();
          }
          if (kluch == 2) {                                               // выводим в порт температуру
#ifdef SetConnectPC
#ifdef SerialPortPlotter                                                  // готовим данные для вывода на ПК
            sprintf (buf, "$%03d %03d %03d %03d;", int(Output1), int(Output2), tc1, tc2);       // стандартный вывод данных
#else
            sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
#endif
            Serial.print(buf);
#endif
          }
          if (kluch == 3) {                                               // работа защиты от отвала термопары
            /*                if (ArObsi[0] == 1) {   // проверка защиты низа разрешена
                              if (Gvard_If()) {
                                F_Alarm = true;
                                ChangeState(REFLOW_STATE_BRAZE_STOP,1);   // завершение пайки
                                }
                            }
                          } */
          }
        }  break;
      }

    case REFLOW_STATE_BRAZE_STOP:   {                                     // STOP процесса пайки
        if (updateScreen) {
          //          digitalWrite(P1_PIN, SetReleOFF);
          //          digitalWrite(P2_PIN, SetReleOFF);
          //          digitalWrite(P3_PIN, SetReleOFF);
          //          digitalWrite(P4_PIN, SetReleOFF);
          TopStart = false;
          Output2 = 0;
          Output1 = 0;

          lcd.setCursor(0, 0);
          lcd.print(F("  STOP Profile   "));


#ifdef SetInterrupt
          detachInterrupt(SetInterrupt);                                 // остановить порт прерывания(0 или 1) 2й или 3й цифровой пин
#else
          MsTimer2::stop();
#endif
          digitalWrite(RelayPin1, LOW);
          digitalWrite(RelayPin2, LOW);

          if (F_Alarm) {
            //              myGLCD.setColor(VGA_RED);
            //              myGLCD.setFont(BigFontRus);
            //              myGLCD.textRus(F(" АВАРИЯ! СТОП ПРОФИЛЯ     "), 20, 302);
          }
          else {
            //              myGLCD.setColor(VGA_LIME);
            //              myGLCD.setFont(BigFontRus);
            //              myGLCD.textRus(F("         СТОП ПРОФИЛЯ           "), 5, 302);
          }


          tone(buzzerPin, 945, 500);                                     // звуковой сигнал
          delay(150);
          tone(buzzerPin, 645, 500);                                     // звуковой сигнал
          updateScreen = false;
        }
        if (millis() > nextRead1)     {                                  // график температуры nextRead1
          nextRead1 = millis() + SENSOR_SAMPLING_TIME;
          if (chast < 450) Count1++;
          chast = Count1 / 4;
          kluch =  Count1 % 4;
          TempRead();                                                    // считываем температуру
          if (kluch == 0) {
            TempDisplay();
            diPrin(17, 0, chast);
#ifdef SetConnectPC
#ifdef SerialPortPlotter                                                 // готовим данные для вывода на ПК
            sprintf (buf, "$%03d %03d %03d %03d;", int(Output1), int(Output2), tc1, tc2);       // стандартный вывод данных
#else
            sprintf (buf, "OK%03d%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2, currentProfile);
#endif
            Serial.print(buf);
#endif
          }
        }
        break;
      }

  }                                                                      //здесь закончился switch

} // end loop
